const himitsu = browser.runtime.connectNative("himitsu");

let conns = [];
let requests = [];
let activeTab = null;

browser.browserAction.onClicked.addListener(tab => {
	activeTab = tab;
	browser.browserAction.setPopup({
		popup: browser.runtime.getURL("content/popup.html"),
	});
	browser.browserAction.openPopup();
});

himitsu.onMessage.addListener(msg => {
	const req = requests.shift();
	switch (req.kind) {
	case "popup":
		if (msg.length === 0) {
			// TODO: New key
		} else {
			req.conn.postMessage({
				action: "select-key",
				keys: msg,
			});
		}
		break;
	case "page":
		browser.tabs.sendMessage(req.tabId, {
			action: "autofill",
			key: msg[0],
		});
		break;
	}
});

function encodeQuery(q) {
	let result = "";
	for (let i = 0; i < q.length; i += 1) {
		const pair = q[i];
		result += pair.key;
		if (pair.private) {
			result += "!";
		}
		if (pair.optional) {
			result += "?";
		}
		if (pair.value != null) {
			result += "=";
			result += shquote(pair.value);
		}
		if (i + 1 < q.length) {
			result += " ";
		}
	};
	return result;
}

function onMessage(conn, msg) {
	switch (msg.action) {
	case "init-popup":
		initPopup(conn);
		break;
	case "autofill":
		requests.push({ kind: "page", tabId: activeTab.id });
		himitsu.postMessage({
			op: "query",
			query: encodeQuery(msg.key),
			decrypt: true,
		});
		break;
	default:
		console.log(`Warning: unknown background action ${msg.action}`);
	}
}

async function initPopup(conn) {
	const tab = activeTab;
	const url = new URL(tab.url);
	browser.browserAction.setPopup({ popup: null });
	requests.push({ kind: "popup", conn });
	himitsu.postMessage({
		op: "query",
		query: `proto=web host=${url.hostname}`,
		decrypt: false,
	});
}

browser.runtime.onConnect.addListener(conn => {
	conn.onMessage.addListener(msg => onMessage(conn, msg));
	conn.onDisconnect.addListener(() => {
		conns = conns.filter(c => c.conn !== conn);
	});
	conns.push({
		conn,
	});
});

// https://github.com/rgov/node-shlex
// MIT licensed
// Copyright (c) 2018 Ryan Govostes
function shquote(s) {
  if (s === '') { return '\'\'' }

  const unsafeRe = /[^\w@%\-+=:,./]/
  if (!unsafeRe.test(s)) { return s }

  return ('\'' + s.replace(/('+)/g, '\'"$1"\'') + '\'').replace(/^''|''$/g, '')
}
